﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm


namespace simpledb.file
{
    /// <summary>
    /// A reference to a disk block.
    /// A Block object consists of a filename and a block number.
    /// It does not hold the contents of the block;
    /// instead, that is the job of a <see cref="Page"/> object.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class Block
    {
        private string filename;
        private int blknum;

        /// <summary>
        /// Constructs a block reference 
        /// for the specified filename and block number.
        /// </summary>
        /// <param name="filename">the name of the file</param>
        /// <param name="blknum">the block number</param>
        public Block(string filename, int blknum)
        {
            this.filename = filename;
            this.blknum = blknum;
        }

        /// <summary>
        /// Returns the name of the file where the block lives.
        /// </summary>
        /// <returns>the filename</returns>
        public string fileName()
        {
            return filename;
        }

        /// <summary>
        /// Returns the location of the block within the file.
        /// </summary>
        /// <returns>the block number</returns>
        public int number()
        {
            return blknum;
        }

        /// <summary>
        /// Determines whether the specified Block object is equal to the current Block object.
        /// </summary>
        /// <param name="obj">The Block object to compare with the current Block object. </param>
        /// <returns>true if the specified Block object is equal to the current Block object; otherwise, false.</returns>
        public override bool Equals(object obj)
        {
            Block blk = (Block)obj;
            return filename.Equals(blk.filename) && blknum == blk.blknum;
        }

        /// <summary>
        /// Returns a String that represents the current Block object.
        /// </summary>
        /// <returns>A String that represents the current Block object.</returns>
        public override string ToString()
        {
            return "[file " + filename + " ,block " + blknum + "]";
        }

        /// <summary>
        /// Serves as a hash function for a particular type. 
        /// </summary>
        /// <returns>A hash code for the current Block object.</returns>
        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }
    }
}
