// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;
using simpledb.query;

namespace simpledb.index
{
    /// <summary>
    /// This interface contains methods to traverse an index.
    /// <author>Edward Sciore</author>
    /// </summary>
    public interface Index
    {
        /// <summary>
        /// Positions the index before the first record
        /// having the specified search key.
        /// </summary>
        /// <param name="searchkey">the search key value</param>
        void beforeFirst(Constant searchkey);

        /// <summary>
        /// Moves the index to the next record having the
        /// search key specified in the beforeFirst method. 
        /// Returns false if there are no more such index records.
        /// </summary>
        /// <returns>false if no other index records have the search key</returns>
        bool next();

        /// <summary>
        /// Returns the dataRID value stored in the current index record. 
        /// </summary>
        /// <returns>the dataRID stored in the current index record</returns>
        RID getDataRid();

        /// <summary>
        /// Deletes the index record having the specified
        /// dataval and dataRID values.
        /// </summary>
        /// <param name="dataval">the dataval of the deleted index record</param>
        /// <param name="datarid">the dataRID of the deleted index record</param>
        void insert(Constant dataval, RID datarid);

        /// <summary>
        /// Deletes the index record having the specified
        /// dataval and dataRID values.
        /// </summary>
        /// <param name="dataval">the dataval of the deleted index record</param>
        /// <param name="datarid">the dataRID of the deleted index record</param>
        void delete(Constant dataval, RID datarid);

        /// <summary>
        /// Closes the index 
        /// </summary>
        void close();
    }
}
