﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.record;
using simpledb.metadata;
using simpledb.query;

namespace simpledb.index.query
{
    /// <summary>
    /// The Plan class corresponding to the <i>indexselect</i>
    /// relational algebra operator.
    /// </summary>
    /// <author>Edward Sciore</author> 
    public class IndexSelectPlan : Plan
    {
        private Plan p;
        private IndexInfo ii;
        private Constant val;

        /// <summary>
        /// Creates a new indexselect node in the query tree
        /// for the specified index and selection constant.
        /// </summary>
        /// <param name="p">the input table</param>
        /// <param name="ii">information about the index</param>
        /// <param name="val">the selection constant</param>
        /// <param name="tx">the calling transaction</param>
        public IndexSelectPlan(Plan p, IndexInfo ii, Constant val, Transaction tx)
        {
            this.p = p;
            this.ii = ii;
            this.val = val;
        }

        /// <summary>
        /// Creates a new indexselect scan for this query
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/>
        public Scan open()
        {
            TableScan ts = (TableScan)p.open();
            Index idx = ii.open();
            return new IndexSelectScan(idx, val, ts);
        }

        /// <summary>
        /// /// Estimates the number of block accesses to compute the 
        /// index selection, which is the same as the 
        /// index traversal cost plus the number of matching data records.
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        /// </summary>
        public int blocksAccessed()
        {
            return ii.blocksAccessed() + recordsOutput();
        }

        /// <summary>
        /// Estimates the number of output records in the index selection,
        /// which is the same as the number of search key values
        /// for the index.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/>
        public int recordsOutput()
        {
            return ii.recordsOutput();
        }

        /// <summary>
        /// Returns the distinct values as defined by the index.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.distinctValues"/>
        public int distinctValues(string fldname)
        {
            return ii.distinctValues(fldname);
        }

        /// <summary>
        /// Returns the schema of the data table.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.schema"/>
        public Schema schema()
        {
            return p.schema();
        }
    }
}
