// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.file;

namespace simpledb.log
{
    /// <summary>
    ///  A class that provides the ability to read the values of
    /// a log record.
    /// The class has no idea what values are there.
    /// Instead, the methods <see cref="nextInt"/> 
    /// and <see cref="nextString"/>  read the values 
    /// sequentially.
    /// Thus the client is responsible for knowing how many values
    /// are in the log record, and what their types are.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class BasicLogRecord
    {
        private Page pg;
        private int pos;

        /// <summary>
        /// A log record located at the specified position of the specified page.
        /// This constructor is called exclusively by
        /// <see cref=" LogIterator.MoveNext"/>.
        /// </summary>
        /// <param name="pg"> the page containing the log record</param>
        /// <param name="pos">the position of the log record </param>  
        public BasicLogRecord(Page pg, int pos)
        {
            this.pg = pg;
            this.pos = pos;
        }

        /// <summary>
        /// Returns the next value of the current log record, 
        /// assuming it is an integer.
        /// </summary>
        /// <returns>the next value of the current log record</returns> 
        public int nextInt()
        {
            int result = pg.getInt(pos);
            pos += Page.INT_SIZE;
            return result;
        }

        /// <summary>
        ///  Returns the next value of the current log record, 
        /// assuming it is a string.
        /// </summary>
        /// <returns>the next value of the current log record</returns> 
        public string nextString()
        {
            string result = pg.getString(pos);
            pos += Page.STR_SIZE(result.Length);
            return result;
        }
    }
}
