﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    /// An object that holds the values of the grouping fields
    /// for the current record of a scan.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class GroupValue
    {
        private Dictionary<string, Constant> vals;

        /// <summary>
        ///  Creates a new group value, given the specified scan
        /// and list of fields.
        /// The values in the current record of each field are 
        /// stored.
        /// </summary>
        /// <param name="s">a scan</param>  
        /// <param name="fields">the list of fields</param>  
        public GroupValue(Scan s, List<string> fields)
        {
            vals = new Dictionary<string, Constant>();
            foreach (string fldname in fields)
                vals.Add(fldname, s.getVal(fldname));
        }

        /// <summary>
        ///  Returns the Constant value of the specified field in the group.
        /// <param name="fldname">the name of a field</param>  
        /// </summary>
        /// <returns>the value of the field in the group</returns> 
        public Constant getVal(string fldname)
        {
            return (Constant)vals[fldname];
        }

        /// <summary>
        /// Two GroupValue objects are equal if they have the same values
        /// for their grouping fields. 
        /// </summary>
        public override bool Equals(object obj)
        {
            GroupValue gv = (GroupValue)obj;
            foreach (string fldname in vals.Keys)
            {
                Constant v1 = (Constant)vals[fldname];
                Constant v2 = gv.getVal(fldname);
                if (!v1.Equals(v2))
                    return false;
            }
            return true;
        }

        /// <summary>
        /// The hashcode of a GroupValue object is the sum of the
        /// hashcodes of its field values. 
        /// </summary>
        public override int GetHashCode()
        {
            int hashVal = 0;
            foreach (Constant c in vals.Values)
                hashVal += c.GetHashCode();
            return hashVal;
        }
    }
}
