﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.record;
using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    /// A class that creates temporary tables.
    /// A temporary table is not registered in the catalog.
    /// The class therefore has a method getTableInfo to return the 
    /// table's metadata. 
    /// </summary>
    /// <author>Edward Sciore</author>
    public class TempTable
    {
        private static int nextTableNum = 0;
        private TableInfo ti;
        private Transaction tx;
        private static object threadLock = new object();

        /// <summary>
        /// Allocates a name for for a new temporary table
        /// having the specified schema.
        /// </summary>
        /// <param name="sch">The new table's schema</param>
        /// <param name="tx">The calling transaction</param>
        public TempTable(Schema sch, Transaction tx)
        {
            string tblname = nextTableName();
            ti = new TableInfo(tblname, sch);
            this.tx = tx;
        }

        /// <summary>
        /// Opens a table scan for the temporary table.
        /// </summary>
        /// <returns>an tablescan object</returns>
        public UpdateScan open()
        {
            return new TableScan(ti, tx);
        }

        /// <summary>
        /// Return the table's metadata.
        /// </summary>
        /// <returns>the table's metadata</returns>
        public TableInfo getTableInfo()
        {
            return ti;
        }

        private static string nextTableName()
        {
            lock (threadLock)
            {
                nextTableNum++;
                return "temp" + nextTableNum;
            }
        }
    }
}
