﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.server;

namespace simpledb.multibuffer
{
    /// <summary>
    /// A class containing static methods,
    /// which estimate the optimal number of buffers
    /// to allocate for a scan.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class BufferNeeds
    {
        /// <summary>
        /// This method considers the various roots
        /// of the specified output size (in blocks),
        /// and returns the highest root that is less than
        /// the number of available buffers.
        /// </summary>
        /// <param name="size">the size of the output file</param>
        /// <returns>the highest number less than the number of available buffers, that is a root of the plan's output size</returns>
        public static int bestRoot(int size)
        {
            int avail = SimpleDB.bufferMgr().available();
            if (avail <= 1)
                return 1;
            int k = int.MaxValue;
            double i = 1.0;
            while (k > avail)
            {
                i++;
                k = (int)System.Math.Ceiling(System.Math.Pow(size, 1 / i));
            }
            return k;
        }

        /// <summary>
        /// This method considers the various factors
        /// of the specified output size (in blocks),
        /// and returns the highest factor that is less than
        /// the number of available buffers.
        /// </summary>
        /// <param name="size">the size of the output file</param>
        /// <returns>the highest number less than the number of available buffers, that is a factor of the plan's output size</returns>
        public static int bestFactor(int size)
        {
            int avail = SimpleDB.bufferMgr().available();
            if (avail <= 1)
                return 1;
            int k = size;
            double i = 1.0;
            while (k > avail)
            {
                i++;
                k = (int)System.Math.Ceiling(size / i);
            }
            return k;
        }
    }
}
