﻿using System.Collections.Generic;
using simpledb.query;

namespace simpledb.parse
{
    /// <summary>
    /// Data for the SQL <i>update</i> statement.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class ModifyData
    {
        private string tblname;
        private string fldname;
        private Expression newval;
        private Predicate i_pred = new Predicate();

        /// <summary>
        /// Saves the table name, the modified field and its new value, and the predicate.
        /// </summary>
        /// <param name="tblname"></param>
        /// <param name="fldname"></param>
        /// <param name="newval"></param>
        /// <param name="pred"></param>
        public ModifyData(string tblname, string fldname, Expression newval,
            Predicate pred)
        {
            this.tblname = tblname;
            this.fldname = fldname;
            this.newval = newval;
            this.i_pred = pred;
        }

        /// <summary>
        /// Returns the name of the affected table.
        /// </summary>
        /// <returns>the name of the affected table</returns>
        public string tableName()
        {
            return tblname;
        }

        /// <summary>
        /// Returns the field whose values will be modified
        /// </summary>
        /// <returns>the name of the target field</returns>
        public string targetField()
        {
            return fldname;
        }

        /// <summary>
        /// Returns an expression.
        /// Evaluating this expression for a record produces
        /// the value that will be stored in the record's target field.
        /// </summary>
        /// <returns>the target expression</returns>
        public Expression newValue()
        {
            return newval;
        }

        /// <summary>
        /// Returns the predicate that describes which
        /// records should be modified.
        /// </summary>
        /// <returns>the modification predicate</returns>
        public Predicate pred()
        {
            return i_pred;
        }
    }
}
