﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.tx;
using simpledb.query;
using simpledb.parse;
using simpledb.server;

namespace simpledb.planner
{
    /// <summary>
    /// The simplest, most naive query planner possible.
    /// </summary>
    /// <author>Edward Sciore</author> 
    class BasicQueryPlanner : QueryPlanner
    {
        /// <summary>
        /// Creates a query plan as follows.  It first takes
        /// the product of all tables and views; it then selects on the predicate;
        /// and finally it projects on the field list. 
        /// </summary>
        public Plan createPlan(QueryData data, Transaction tx)
        {
            //step 1: Create A plan for each mentioned table or view
            List<Plan> plans = new List<Plan>();
            foreach (string tblname in data.tables())
            {
                string viewdef = SimpleDB.mdMgr().getViewDef(tblname, tx);
                if (viewdef != null)
                    plans.Add(SimpleDB.planner().createQueryPlan(viewdef, tx));
                else
                    plans.Add(new TablePlan(tblname, tx));
            }

            // step2: Create the product of all tables plans
            Plan p = plans[0];
            plans.RemoveAt(0);
            foreach (Plan nextplan in plans)
                p = new ProductPlan(p, nextplan);

            // step3: add a selectction for the predicate
            p = new SelectPlan(p, data.pred());

            // step4: project on the field names
            p = new ProjectPlan(p, data.fields());
            return p;
        }
    }
}
