﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.query;
using simpledb.parse;

namespace simpledb.planner
{
    /// <summary>
    /// The object that executes SQL statements.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class Planner
    {
        private QueryPlanner qplanner;
        private UpdatePlanner uplanner;

        /// <summary>
        /// Initializes a new instance of the <see cref="Planner"/> class.
        /// </summary>
        /// <param name="qplanner">The query planner.</param>
        /// <param name="uplanner">The update planner.</param>
        public Planner(QueryPlanner qplanner, UpdatePlanner uplanner)
        {
            this.qplanner = qplanner;
            this.uplanner = uplanner;
        }

        /// <summary>
        /// Creates a plan for an SQL select statement, using the supplied planner.
        /// </summary>
        /// <param name="qry">the SQL query string</param>
        /// <param name="tx">the transaction</param>
        /// <returns>the scan corresponding to the query plan</returns>
        public Plan createQueryPlan(string qry, Transaction tx)
        {
            Parser parse = new Parser(qry);
            QueryData data = parse.query();
            return qplanner.createPlan(data, tx);
        }

        /// <summary>
        /// Executes an SQL insert, delete, modify, or
        /// create statement.
        /// The method dispatches to the appropriate method of the
        /// supplied update planner,
        /// depending on what the parser returns.
        /// </summary>
        /// <param name="cmd">the SQL update string</param>
        /// <param name="tx">the transaction</param>
        /// <returns>an integer denoting the number of affected records</returns>
        public int executeUpdate(string cmd, Transaction tx)
        {
            Parser parser = new Parser(cmd);
            object obj = parser.updateCmd();
            if (obj is InsertData)
                return uplanner.executeInsert((InsertData)obj, tx);
            else if (obj is ModifyData)
                return uplanner.executeModify((ModifyData)obj, tx);
            else if (obj is DeleteData)
                return uplanner.executeDelete((DeleteData)obj, tx);
            else if (obj is CreateTableData)
                return uplanner.executeCreateTable((CreateTableData)obj, tx);
            else if (obj is CreateIndexData)
                return uplanner.executeCreateIndex((CreateIndexData)obj, tx);
            else if (obj is CreateViewData)
                return uplanner.executeCreateView((CreateViewData)obj, tx);
            else
                return 0;
        }
    }
}
