﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The interface corresponding to SQL expressions.
    /// </summary>
    /// <author>Edward Sciore</author>
    public interface Expression
    {
        /// <summary>
        /// Returns true if the expression is a constant.
        /// </summary>
        /// <returns>true if the expression is a constant</returns> 
        bool isConstant();

        /// <summary>
        ///  Returns true if the expression is a field reference.
        /// </summary>
        /// <returns>true if the expression denotes a field</returns>
        bool isFieldName();

        /// <summary>
        /// Returns the constant corresponding to a constant expression.
        /// Throws an exception if the expression does not
        /// denote a constant.
        /// </summary>
        /// <returns>the expression as a constant</returns> 
        Constant asConstant();

        /// <summary>
        /// Returns the field name corresponding to a constant expression.
        /// Throws an exception if the expression does not
        /// denote a field.
        /// </summary>
        /// <returns>the expression as a field name</returns> 
        string asFieldName();

        /// <summary>
        /// Evaluates the expression with respect to the
        /// current record of the specified scan. 
        /// </summary>
        /// <param name="s">the scan</param>  
        /// <returns>the value of the expression, as a Constant</returns> 
        Constant evaluate(Scan s);

        /// <summary>
        /// Determines if all of the fields mentioned in this expression
        /// are contained in the specified schema.      
        /// </summary>
        /// <param name="sch">the schema</param>  
        /// <returns>true if all fields in the expression are in the schema</returns> 
        bool appliesTo(Schema sch);
    }
}
