﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm


namespace simpledb.query
{
    /// <summary>
    /// The class that wraps strings as database constants.
    /// </summary> 
    /// /// <author>Edward Sciore</author>
    public class StringConstant : Constant
    {
        private string val;

        /// <summary>
        ///  Create a constant by wrapping the specified string.
        /// </summary>
        /// <param name="s">the string value</param>
        public StringConstant(string s)
        {
            val = s;
        }

        /// <summary>
        /// Unwraps the string and returns it.
        /// <seealso cref="simpledb.query.Constant.asCsharpVal"/> 
        /// </summary>
        public object asCsharpVal()
        {
            return val;
        }

        /// <summary>
        /// Compares this instance to a specified object and returns an indication of their relative values.
        /// </summary>
        /// <param name="c">An object to compare</param>
        /// <returns>A signed number indicating the relative values of this instance and value.</returns>
        public int CompareTo(Constant c)
        {
            StringConstant sc = (StringConstant)c;
            return val.CompareTo(sc.val);
        }

        /// <summary>
        /// Determines whether the specified <see cref="System.Object"/> is equal to this instance.
        /// </summary>
        /// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
        /// <returns>
        /// 	<c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">
        /// The <paramref name="obj"/> parameter is null.
        /// </exception>
        public override bool Equals(object obj)
        {
            StringConstant sc = (StringConstant)obj;
            return sc != null && val.Equals(sc.val);
        }

        /// <summary>
        /// Returns a hash code for this instance.
        /// </summary>
        /// <returns>
        /// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
        /// </returns>
        public override int GetHashCode()
        {
            return val.GetHashCode();
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "'"+val+"'";
        }
    }
}
