﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The interface implemented by all updateable scans.
    /// </summary> 
    /// <author>Edward Sciore</author>
    public interface UpdateScan : Scan
    {
        /// <summary>
        ///  Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname"> the name of the field</param>
        /// <param name="val">the new value, expressed as a Constant</param>
        void setVal(string fldname, Constant val);

        /// <summary>
        /// /// Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="val">the new integer value</param>
        void setInt(string fldname, int val);

        /// <summary>
        /// /// Modifies the field value of the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <param name="val">the new string value</param>
        void setString(string fldname, string val);

        /// <summary>
        /// Inserts a new record somewhere in the scan.
        /// </summary>
        void insert();

        /// <summary>
        /// Deletes the current record from the scan.
        /// </summary>
        void delete();

        /// <summary>
        ///  Returns the RID of the current record.
        /// </summary>
        /// <returns>the RID of the current record</returns>
        RID getRid();

        /// <summary>
        ///  Positions the scan so that the current record has
        /// the specified RID.
        /// </summary>
        /// <param name="RID">the RID of the desired record</param>
        void moveToRid(RID RID);
    }
}
