﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.file;
using simpledb.buffer;

namespace simpledb.record
{
    /// <summary>
    /// An object that can format a page to look like a block of 
    /// empty records.
    /// </summary>
    /// <author>Edward Sciore</author>
    class RecordFormatter : PageFormatter
    {
        private TableInfo ti;

        /// <summary>
        /// Creates a formatter for a new page of a table.
        /// </summary>
        /// <param name="ti">the table's metadata</param>
        public RecordFormatter(TableInfo ti)
        {
            this.ti = ti;
        }

        /// <summary>
        /// Formats the page by allocating as many record slots
        /// as possible, given the record length.
        /// Each record slot is assigned a flag of EMPTY.
        /// Each integer field is given a value of 0, and
        /// each string field is given a value of "".
        /// </summary>
        /// <seealso cref="simpledb.buffer.PageFormatter.format"/> 
        public void format(Page page)
        {
            int recsize = ti.recordLength() + Page.INT_SIZE;
            for (int pos = 0; pos + recsize <= Page.BLOCK_SIZE; pos += recsize)
            {
                page.setInt(pos, RecordPage.EMPTY);
                makeDefaultRecord(page, pos);
            }
        }

        private void makeDefaultRecord(Page page, int pos)
        {
            foreach (string fldname in ti.schema().fields())
            {
                int offset = ti.offset(fldname);
                if (ti.schema().type(fldname) == Schema.INTEGER)
                    page.setInt(pos + Page.INT_SIZE + offset, 0);
                else
                    page.setString(pos + Page.INT_SIZE + offset, "");
            }
        }
    }
}
