// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.log;

namespace simpledb.tx.recovery
{
    /// <summary>
    /// The CHECKPOINT log record.
    /// </summary>
    /// <author>Edward Sciore</author> 
    class CheckpointRecord : LogRecord
    {
        /// <summary>
        /// Creates a quiescent checkpoint record 
        /// </summary>
        public CheckpointRecord() { }

        /// <summary>
        /// Creates a log record by reading no other values 
        /// from the basic log record.
        /// </summary>
        /// <param name="rec">the basic log record</param>
        public CheckpointRecord(BasicLogRecord rec) { }

        /// <summary>
        /// Writes a checkpoint record to the log.
        /// This log record contains the CHECKPOINT operator,
        /// and nothing else.
        /// </summary>
        /// <returns>the LSN of the last log value</returns>
        public override int writeToLog()
        {
            object[] rec = new object[] { CHECKPOINT };
            return logMgr.append(rec);
        }

        /// <summary>
        /// returns the log record's type
        /// </summary>
        /// <returns>the log record's type</returns>
        public override int op()
        {
            return CHECKPOINT;
        }

        /// <summary>
        /// Checkpoint records have no associated transaction,
        /// and so the method returns a "dummy", negative txid
        /// </summary>
        /// <returns>the log record's transaction id</returns>
        public override int txNumber()
        {
            return -1;
        }

        /// <summary>
        /// Does nothing, because a checkpoint record
        /// contains no undo information. 
        /// </summary>
        public override void undo(int txnum) { }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "<CHECKPOINT>";
        }
    }
}
