// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.server;
using simpledb.log;

namespace simpledb.tx.recovery
{
    /// <summary>
    /// The interface implemented by each type of log record.
    /// </summary>
    /// <author>Edward Sciore</author> 
    abstract class LogRecord
    {
        /// <summary>
        /// The six different types of log record 
        /// </summary>
        public const int CHECKPOINT = 0, START = 1, COMMIT = 2, ROLLBACK = 3, SETINT = 4, SETSTRING = 5;

        /// <summary>
        /// the simpledb log manager
        /// </summary>
        public static readonly LogMgr logMgr = SimpleDB.logMgr();

        /// <summary>
        /// Writes the record to the log and returns its LSN
        /// </summary>
        /// <returns>the LSN of the record in the log</returns>
        public abstract int writeToLog();


        /// <summary>
        /// Returns the log record's type.
        /// </summary>
        /// <returns>the log record's type</returns>
        public abstract int op();

        /// <summary>
        /// Returns the transaction id stored with
        /// the log record.
        /// </summary>
        /// <returns>the log record's transaction id</returns>
        public abstract int txNumber();

        /// <summary>
        /// Undoes the operation encoded by this log record.
        /// The only log record types for which this method
        /// does anything interesting are SETINT and setString.
        /// </summary>
        /// <param name="txnum">the id of the transaction that is performing the undo</param>
        public abstract void undo(int txnum);
    }
}
