// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections;
using System.Collections.Generic;
using simpledb.log;
using simpledb.server;

namespace simpledb.tx.recovery
{
    /// <summary>
    /// A class that provides the ability to read records
    /// from the log in reverse order.
    /// Unlike the similar class 
    /// <see cref="simpledb.log.LogIterator"/>,  
    /// this class understands the meaning of the log records.
    /// </summary>
    /// <author>Edward Sciore</author> 
    class LogRecordIterator : IEnumerator<LogRecord>
    {
        private IEnumerator<BasicLogRecord> iter = SimpleDB.logMgr().GetEnumerator();

        /// <summary>
        /// Constructs a log record from the values in the 
        /// current basic log record.
        /// The method first reads an integer, which denotes
        /// the type of the log record.  Based on that type,
        /// the method calls the appropriate LogRecord constructor
        /// to read the remaining values.
        /// </summary>
        public LogRecord Current
        {
            get
            {
                BasicLogRecord rec = iter.Current;
                int op = rec.nextInt();
                switch (op)
                {
                    case LogRecord.CHECKPOINT:
                        return new CheckpointRecord(rec);
                    case LogRecord.START:
                        return new StartRecord(rec);
                    case LogRecord.COMMIT:
                        return new CommitRecord(rec);
                    case LogRecord.ROLLBACK:
                        return new RollbackRecord(rec);
                    case LogRecord.SETINT:
                        return new SetIntRecord(rec);
                    case LogRecord.SETSTRING:
                        return new SetStringRecord(rec);
                    default:
                        return null;
                }
            }
        }

        object IEnumerator.Current
        {
            get
            {
                return Current;
            }
        }

        /// <summary>
        /// Advances to the next log record in reverse order.
        /// </summary>
        /// <returns>
        /// true if successfully advanced, otherwise false
        /// </returns>
        public bool MoveNext()
        {
            return iter.MoveNext();
        }

        /// <summary>
        /// Sets the enumerator to its initial position.
        /// </summary>
        public void Reset()
        {
            iter.Reset();
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
        }
    }
}
